/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.test.ws.jaxws.jbws1869;

import junit.framework.Test;
import org.jboss.wsf.test.JBossWSTest;
import org.jboss.wsf.test.JBossWSTestSetup;

import javax.xml.namespace.QName;
import javax.xml.ws.Service;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;

/**
 * Support stream based RequestHandler API
 */
public class JBWS1869TestCase extends JBossWSTest
{
   private final static String TNS =            "http://jbws1869.jaxws.ws.test.jboss.org/";
   private final static QName portName =        new QName(TNS, "JBWS1869EndpointImplPort");
   private final static QName serviceName =     new QName(TNS, "JBWS1869EndpointImplService");

   private final static String REQUEST = "<?xml version=\"1.0\" ?><S:Envelope xmlns:S=\"http://schemas.xmlsoap.org/soap/envelope/\"><S:Body><ns2:echo xmlns:ns2=\"http://jbws1869.jaxws.ws.test.jboss.org/\"><arg0>StreamBasedInvocation</arg0></ns2:echo></S:Body></S:Envelope>";
   private final static String RESPONSE = "<S:Envelope xmlns:S=\"http://schemas.xmlsoap.org/soap/envelope/\"><S:Body><ns2:echoResponse xmlns:ns2=\"http://jbws1869.jaxws.ws.test.jboss.org/\"><return>StreamBasedInvocation</return></ns2:echoResponse></S:Body></S:Envelope>";

   public static Test suite()
   {
      return new JBossWSTestSetup(JBWS1869TestCase.class, "jaxws-jbws1869.war");
   }

   /**
    * Go through {@link org.jboss.wsf.spi.invocation.RequestHandler#handleHttpRequest(org.jboss.wsf.spi.deployment.Endpoint, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, javax.servlet.ServletContext)} 
    * @throws Exception
    */
   public void testServletInvocation() throws Exception
   {
      Service service = Service.create(
        new URL("http://"+getServerHost()+":8080/jaxws-jbws1869?wsdl"), serviceName
      );
      JBWS1869Endpoint port = service.getPort(portName,  JBWS1869Endpoint .class);

      String res = port.echo("ServletInvocation");
      assertEquals(res, "ServletInvocation");
   }

   /**
    * Go through {@link org.jboss.wsf.spi.invocation.RequestHandler#handleRequest(org.jboss.wsf.spi.deployment.Endpoint, java.io.InputStream, java.io.OutputStream, org.jboss.wsf.spi.invocation.InvocationContext)}
    * @throws Exception
    */
   public void testStreamBasedInvocation() throws Exception
   {     
      // Send data
      URL url = new URL("http://"+getServerHost()+":8080/jaxws-jbws1869/delegate");
      HttpURLConnection conn = (HttpURLConnection)url.openConnection();
      conn.setDoOutput(true);
      conn.setRequestProperty("Content-Type", "text/xml");


      OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
      wr.write(REQUEST);
      wr.flush();
      
      // Get the response
      StringBuffer sb = new StringBuffer();
      BufferedReader rd = new BufferedReader(new InputStreamReader(conn.getInputStream()));
      String line;
      while ((line = rd.readLine()) != null)
      {
         sb.append(line);
      }
      wr.close();
      rd.close();

      // response assertion
      assertTrue(sb.toString().indexOf(RESPONSE) != -1);      
   }
}
